/*--------------------------------------------------------------
  Program:      PZEM-004 & Arduino Nano + Modbus RTU Slave
  Description:  Connection and communication Serial power meter Active PZEM-004
                Peacefair with Arduino Nano as Slave Modbus RTU serial RS232
                Conexión y comunicación Serial medidor de potencia Activa PZEM-004
                Peacefair con Arduino Nano como Esclavo Modbus RTU serie RS232.
  
  Hardware:     Arduino Nano ATmega328
                PZEM-004
                Converter RS232/TTL
                
  Software:     Arduino IDE v1.8.3
  
  Date:         31 May 2018
   
  Modified or Created:       PDAControl   http://pdacontroles.com   http://pdacontrolen.com
  Complete Tutorial English: 
  
  Tutorial Completo Español: 
  
  Based: PZEM004T library for olehs                          https://github.com/olehs/PZEM004T

--------------------------------------------------------------*/

#include <modbus.h>
#include <modbusDevice.h>
#include <modbusRegBank.h>
#include <modbusSlave.h>

#include <SoftwareSerial.h> // Arduino IDE <1.6.6
#include <PZEM004T.h> //https://github.com/olehs/PZEM004T

PZEM004T pzem(10,11);  // (RX,TX) connect to TX,RX of PZEM                                 
IPAddress ip(192,168,1,1);

float v_mb =0;          
float i_mb =0;          
float p_mb =0;        
float e_mb =0;

modbusDevice regBank;
modbusSlave slave;
 
void setup()
{   

 pzem.setAddress(ip);

//Assign the modbus device ID.  
  regBank.setId(1);

// Holding registers
    
  regBank.add(40010);   
  regBank.add(40011);  
  regBank.add(40012);  
  regBank.add(40013);  
  regBank.add(40014);  
 

  slave._device = &regBank;  

  Serial.begin(9600);
  delay(100);
// Initialize the serial port for coms at 9600 baud  
  slave.setBaud(9600);
  delay(100);
  Serial.println("Connected "); 
  Serial.println("Modbus RTU Slave Online");  
}

void loop()
{    
  
        float v = pzem.voltage(ip);    
        if(v >= 0.0){   v_mb =v;      } //V  
        float c = pzem.current(ip);
        if(c >= 0.0){   i_mb= c;       } //A                                                                                                             
        float p = pzem.power(ip);
        if(p >= 0.0){   p_mb=p;       } //kW        
        float e = pzem.energy(ip);          
        if(e >= 0.0){  e_mb =e;        } //kWh   
        

        regBank.set(40010,  random(1, 100));        
        regBank.set(40011,   (v_mb *  10));        
        regBank.set(40012,   (i_mb *  10));              
        regBank.set(40013,   (p_mb *  10));              
        regBank.set(40014,   (e_mb));
         
         slave.run();  
 

}

